@@ -40,24 +40,26 @@ Additionally, there are methods for changing the i2c address (i2c units
 only), setting tap sensitivity, and controlling the LED.
 """
 
-import hashlib
-import os, sys
+import distutils.sysconfig
 import errno
-
+import hashlib
+import os
+import sys
+import typing as t
 from ctypes import *
 
-from typing import Union
-from typing import Any, AnyStr, NoReturn, Optional, Type
-from typing import List, Mapping, Tuple
-
-import distutils.sysconfig
-
-from .exceptions import VerificationError, ZymkeyLibraryError, ZymkeyTimeoutError
+from .exceptions import (VerificationError, ZymkeyLibraryError,
+                         ZymkeyTimeoutError)
 from .settings import ZYMKEY_LIBRARY_PATH
 from .utils import is_string
 from .zka import zkalib
 
-__all__ = ["Zymkey", "client"]
+__all__ = [
+    "Zymkey",
+    "client",
+    "RecoveryStrategy",
+    "RecoveryStrategyBip39",
+]
 
 CLOUD_ENCRYPTION_KEY: str = "cloud"
 """The constant used to refer to the cloud encryption key."""
@@ -65,10 +67,10 @@ CLOUD_ENCRYPTION_KEY: str = "cloud"
 ZYMKEY_ENCRYPTION_KEY: str = "zymkey"
 """The constant used to refer to the zymkey encryption key."""
 
-ENCRYPTION_KEYS: Tuple[str, ...] = (CLOUD_ENCRYPTION_KEY, ZYMKEY_ENCRYPTION_KEY)
+ENCRYPTION_KEYS: t.Tuple[str, ...] = (CLOUD_ENCRYPTION_KEY, ZYMKEY_ENCRYPTION_KEY)
 """A tuple containing the various encryption key constants."""
 
-keyTypes: Mapping[str, int] = {
+keyTypes: t.Mapping[str, int] = {
     "secp256r1": 0,
     "nistp256": 0,
     "secp256k1": 1,
@@ -77,7 +79,7 @@ keyTypes: Mapping[str, int] = {
 }
 """A dict containing the available key types."""
 
-kdfFuncTypes: Mapping[str, int] = {
+kdfFuncTypes: t.Mapping[str, int] = {
     "none": 0,
     "rfc5869-sha256": 1,
     "rfc5869-sha512": 2,
@@ -87,6 +89,37 @@ kdfFuncTypes: Mapping[str, int] = {
 """A dict containing the available KDF function types."""
 
 
+class RecoveryStrategy:
+    """The RecoveryStrategy class definition.
+
+    This class specifies the recovery strategy used for wallet generation within Python.
+    Base class strategy is to do no recovery.
+    """
+
+    def __init__(self):
+        """Initialize an instance of RecoveryStrategy."""
+        self.recovery_strategy = "none"
+
+
+class RecoveryStrategyBip39(RecoveryStrategy):
+    """The RecoveryStrategyBip39 class definition.
+
+    This class specifies the Bip39 recovery strategy used for wallet generation within Python.
+    Derived from RecoveryStrategy class.
+    """
+
+    def __init__(self, passphrase: str):
+        """Initialize an instance of RecoveryStrategyBip39.
+
+        Parameters
+        ----------
+        passphrase
+            Passphrase used for bip39 generation. Can be empty string. Must be b64 encoded.
+        """
+        self.passphrase = passphrase
+        self.recovery_strategy = "bip39"
+
+
 class Zymkey(object):
     """The Zymkey class definition.
 
@@ -99,7 +132,7 @@ class Zymkey(object):
     ###@{
 
     def __init__(self):
-        """The class initialization opens and stores an instance of a Zymkey context."""
+        """Initialize an instance of a Zymkey context."""
         self._zk_ctx = c_void_p()
         ret = self._zkOpen(byref(self._zk_ctx))
 
@@ -247,25 +280,20 @@ class Zymkey(object):
 
     def lock(
         self,
-        src: Union[str, bytes],
-        dst: Optional[str] = None,
+        src: t.Union[str, bytes],
+        dst: t.Optional[str] = None,
         encryption_key: str = ZYMKEY_ENCRYPTION_KEY,
-    ) -> Union[None, bytearray]:
+    ) -> t.Union[None, bytearray]:
         """Lock up source (plaintext) data.
 
-        This methods encrupts and signs a block of data.
+        This methods encrypts and signs a block of data.
 
-        The Zymkey has two keys that can be used for locking/unlocking operations,
-        designated as 'shared' and 'one-way':
+        The Zymkey that can be used for locking/unlocking operations
 
         1. The one-way key is meant to lock up data only on the local host
            computer. Data encrypted using this key cannot be exported and
            deciphered anywhere else.
 
-        2. The shared key is meant for publishing data to other sources that
-           have the capability to generate the shared key, such as the Zymbit
-           cloud server.
-
         Parameters
         ----------
         src
@@ -285,7 +313,7 @@ class Zymkey(object):
         encryption_key
             This specifies which key will be used to lock the data. A value of
             'zymbit' (default) specifies that the Zymkey will use the one-way
-            key. A value of 'cloud' specifies that the shared key is used.
+            key.
 
         Returns
         -------
@@ -301,8 +329,8 @@ class Zymkey(object):
         assert encryption_key in ENCRYPTION_KEYS
         use_shared_key = encryption_key == CLOUD_ENCRYPTION_KEY
 
-        dst_path: Optional[bytes] = None
-        dst_data: Optional[Union[c_void_p, int]] = None
+        dst_path: t.Optional[bytes] = None
+        dst_data: t.Optional[t.Union[c_void_p, int]] = None
         if isinstance(dst, str):
             dst_path = dst.encode("utf-8")
         else:
@@ -382,27 +410,22 @@ class Zymkey(object):
 
     def unlock(
         self,
-        src: Union[str, bytes],
-        dst: Optional[str] = None,
+        src: t.Union[str, bytes],
+        dst: t.Optional[str] = None,
         encryption_key: str = ZYMKEY_ENCRYPTION_KEY,
         raise_exception: bool = True,
-    ) -> Union[None, bytearray, NoReturn]:
+    ) -> t.Union[None, bytearray, t.NoReturn]:
         """Unlock source (ciphertext) data.
 
         This method verifies a locked object signature and decrypts the
         associated ciphertext data.
 
-        The Zymkey has two keys that can be used for locking/unlocking operations,
-        designated as 'shared' and 'one-way':
+        The Zymkey has two keys that can be used for locking/unlocking operations
 
         1. The one-way key is meant to lock up data only on the local host
            computer. Data encrypted using this key cannot be exported and
            deciphered anywhere else.
 
-        2. The shared key is meant for publishing data to other sources that
-           have the capability to generate the shared key, such as the Zymbit
-           cloud server.
-
         Parameters
         ----------
         src
@@ -422,7 +445,7 @@ class Zymkey(object):
         encryption_key
             This specifies which key will be used to lock the data. A value of
             'zymbit' (default) specifies that the Zymkey will use the one-way
-            key. A value of 'cloud' specifies that the shared key is used.
+            key.
         raise_exception
             Specifies if an exception should be raised if the signature verification
             of the locked object fails.
@@ -535,7 +558,9 @@ class Zymkey(object):
 
         return self.sign_digest(sha256, slot=slot, return_recid=return_recid)
 
-    def sign_digest(self, sha256: Any, slot: int = 0, return_recid: bool = False) -> bytearray:
+    def sign_digest(
+        self, sha256: t.Any, slot: int = 0, return_recid: bool = False
+    ) -> bytearray:
         """Generate a signature using the Zymkey's ECDSA private key.
 
         Parameters
@@ -572,14 +597,19 @@ class Zymkey(object):
         recovery_id = c_uint()
 
         ret = self._zkGenECDSASigFromDigestWithRecID(
-            self._zk_ctx, src_c_ubyte, slot, byref(dst_data), byref(dst_data_sz), byref(recovery_id)
+            self._zk_ctx,
+            src_c_ubyte,
+            slot,
+            byref(dst_data),
+            byref(dst_data_sz),
+            byref(recovery_id),
         )
         if ret < 0:
             raise AssertionError("bad return code %d" % ret)
         dc = (c_ubyte * dst_data_sz.value).from_address(dst_data.value)
         data_array = bytearray(dc)
 
-        if(return_recid):
+        if return_recid:
             return data_array, recovery_id
         else:
             return data_array
@@ -634,7 +664,7 @@ class Zymkey(object):
 
     def verify_digest(
         self,
-        sha256: Any,
+        sha256: t.Any,
         sig: bytearray,
         raise_exception: bool = True,
         pubkey_slot: int = 0,
@@ -702,10 +732,10 @@ class Zymkey(object):
     def ecdh(
         self,
         local_slot: int,
-        peer_pubkey: Union[List[bytes], int],
+        peer_pubkey: t.Union[t.List[bytes], int],
         kdf_func_type: str = "none",
-        salt: Optional[List[bytes]] = [],
-        info: Optional[List[bytes]] = [],
+        salt: t.Optional[t.List[bytes]] = [],
+        info: t.Optional[t.List[bytes]] = [],
         num_iterations: int = 1,
         peer_pubkey_slot_is_foreign: bool = True,
         derived_key_size: bool = 32,
@@ -716,7 +746,7 @@ class Zymkey(object):
         ----------
         local_slot : int
             The local key slot to use.
-        peer_pubkey : Union[List[bytes], int]
+        peer_pubkey : t.Union[t.List[bytes], int]
             The public key of the peer used to generate the pre-master secret
             against the private key located in `local_slot`. This parameter can
             be a list of `bytes` if the key is provided explicitly or an `int`
@@ -733,9 +763,9 @@ class Zymkey(object):
             * `"rfc5869-sha512"`: RFC5869 with SHA512
             * `"pbkdf2-sha256"`: PBKDF2 with SHA256
             * `"pbkdf2-sha512"`: PBKDF2 with SHA512
-        salt : Optional[List[bytes]]
+        salt : t.Optional[t.List[bytes]]
             A unique identifier for KDF. Ignored for `kdf_func_type='none'`.
-        info : Optional[List[bytes]]
+        info : t.Optional[t.List[bytes]]
             A unique field for rfc5869. Ignore for other KDF types.
         num_iterations : int
             The number of iterations that the KDF should complete.
@@ -944,7 +974,7 @@ class Zymkey(object):
         data_array = bytearray(dc)
         return data_array
 
-    def get_slot_alloc_list(self, foreign: bool = False) -> Tuple[list, int]:
+    def get_slot_alloc_list(self, foreign: bool = False) -> t.Tuple[list, int]:
         """Get a list of the allocated slots in the key store (model >= HSM6).
 
         This method gets a list of the allocated slots in the key store.
@@ -956,7 +986,7 @@ class Zymkey(object):
 
         Returns
         -------
-        Tuple[list, int]
+        t.Tuple[list, int]
             The allocation list and the maximum number of keys
 
         Raises
@@ -1109,7 +1139,11 @@ class Zymkey(object):
     ###@{
 
     def gen_wallet_master_seed(
-        self, key_type, master_gen_key, wallet_name, bip39=False
+        self,
+        key_type,
+        master_gen_key,
+        wallet_name,
+        recovery_strategy=RecoveryStrategy(),
     ):
         """Generates a new master seed for creating a new BIP32 wallet (model >= HSM6).
 
@@ -1127,9 +1161,10 @@ class Zymkey(object):
         wallet_name
             The name of the wallet (string) that this master seed
             is attached to.
-        bip39
-            If true, generate the seed per BIP39 and return the mnemonic
-            string.
+        recovery_strategy
+            RecoveryStrategy() class that defines what strategy to be used
+            {None, Bip39, Slip39[not supported right now]}.
+            Bip39Strategy->passphrase must be b64 encoded.
 
         Returns
         -------
@@ -1138,6 +1173,13 @@ class Zymkey(object):
         """
         master_gen_key_sz = len(master_gen_key)
         master_gen_key_c_ubyte = (c_ubyte * master_gen_key_sz)(*master_gen_key)
+        bip39 = False
+        bip39_passphrase = ""
+
+        if recovery_strategy.recovery_strategy == "bip39":
+            bip39 = True
+            bip39_passphrase = recovery_strategy.passphrase
+
         bip39_mnemonic = c_void_p()
         if bip39:
             mnemonic_ptr = byref(bip39_mnemonic)
@@ -1151,6 +1193,7 @@ class Zymkey(object):
             wallet_name.encode("utf-8"),
             master_gen_key_c_ubyte,
             master_gen_key_sz,
+            bip39_passphrase.encode("utf-8"),
             mnemonic_ptr,
         )
         if ret < 0:
@@ -1189,7 +1232,7 @@ class Zymkey(object):
         return ret
 
     def restore_wallet_master_seed_from_bip39_mnemonic(
-        self, key_type, master_gen_key, wallet_name, bip39_mnemonic
+        self, key_type, master_gen_key, wallet_name, bip39_passphrase, bip39_mnemonic
     ):
         """Restore a wallet's master seed based on a BIP39 mnemonic string (model >= HSM6).
 
@@ -1205,6 +1248,10 @@ class Zymkey(object):
         master_gen_key
             The master generator key used in the derivation of
             the child key.
+        wallet_name
+            Name of the new wallet to be generated.
+        bip39_passphrase
+            Passphrase used to generate the bip39_mnemonic.
         bip39_mnemonic
             The BIP39 mnemonic string.
 
@@ -1223,6 +1270,7 @@ class Zymkey(object):
             wallet_name.encode("utf-8"),
             master_gen_key_c_ubyte,
             master_gen_key_sz,
+            bip39_passphrase.encode("utf-8"),
             bip39_mnemonic.encode("utf-8"),
         )
         if ret < 0:
@@ -1456,6 +1504,9 @@ class Zymkey(object):
             If true, this API returns the time after the next second
             falls. This means that the caller could be blocked up to one second.
             If False, the API returns immediately with the current time reading.
+        Returns
+        -------
+            Time in epoch seconds.
         """
         epoch_sec = c_int()
         ret = self._zkGetTime(self._zk_ctx, byref(epoch_sec), precise)
@@ -1794,8 +1845,8 @@ class Zymkey(object):
         This function sets the threshold at which if the
         battery voltage falls bellow, the action set by
         set_battery_voltage_action will be carried out.
-        The recommended threshold is 2.3V. If this
-        function isn't called 2.5V is assumed by default.
+        The recommended threshold is 2.3V is assumed by default. Threshold
+        must be below 2.5V.
 
         Parameters
         ----------
@@ -2091,6 +2142,7 @@ class Zymkey(object):
             c_char_p,
             c_void_p,
             c_int,
+            c_char_p,
             POINTER(c_void_p),
         ]
 
@@ -2105,6 +2157,7 @@ class Zymkey(object):
             c_void_p,
             c_int,
             c_char_p,
+            c_char_p,
         ]
 
         _zkGenWalletChildKey = zkalib.zkGenWalletChildKey
@@ -2270,7 +2323,7 @@ class Zymkey(object):
     _zkGetSerialNumberString.argtypes = [c_void_p, POINTER(c_void_p)]
 
 
-client: Optional[Zymkey]
+client: t.Optional[Zymkey]
 try:
     client = Zymkey()
 except AssertionError:
