@@ -20,27 +20,27 @@ def main():
 
     check_outfile(outfile)
 
-    if outfile == '-' and command:
-        raise CommandError('cannot specify command when outfile is stdout')
+    if outfile == "-" and command:
+        raise CommandError("cannot specify command when outfile is stdout")
 
     content = vault.unlock(infile)
 
     status = 0
 
     # if the outfile is a dash, print to the screen and exit
-    if outfile == '-':
+    if outfile == "-":
         print(content)
 
         return status
 
-    with open_with_perms(outfile, 'wb', 0o400) as fh:
+    with open_with_perms(outfile, "wb", 0o400) as fh:
         fh.write(content)
 
     try:
         if command:
-            status = subprocess.call([shell, '-c', command])
+            status = subprocess.call([shell, "-c", command])
         else:
-            sys.stderr.write('Hit ctrl+c when ready to remove unlocked file')
+            sys.stderr.write("Hit ctrl+c when ready to remove unlocked file")
             while True:
                 time.sleep(1.0)
     except KeyboardInterrupt:
@@ -49,5 +49,3 @@ def main():
     os.remove(outfile)
 
     return status
-
-
